function [perf predTest] = dpmm_predict(dpmm, classMembers, xTrain, xTest, cTest, existClass,usePrior)

if nargin == 6
    usePrior = 0;
end

opts = dpmm{1}.opts;
opts.test_data  = xTest;

log_prob = zeros(size(xTest,2),length(existClass));

if nargin > 6
    totalsample = 0;
    for c = 1:length(existClass)
        totalsample = totalsample + length(classMembers{c});
    end
end

% Compute the predictive distribution of each class
for c = 1:length(existClass)
    data.given_data = xTrain(:,classMembers{c});
    if usePrior
        log_prob(:,c)   = log(length(classMembers{c})/totalsample) + log_predictive_dist(data, ...
            dpmm{c}.q_of_z, dpmm{c}.hp_posterior, dpmm{c}.hp_prior, opts);
    else
        log_prob(:,c)   = log_predictive_dist(data, ...
            dpmm{c}.q_of_z, dpmm{c}.hp_posterior, dpmm{c}.hp_prior, opts);
    end
end

[dummy activ] = max(log_prob(:,1:length(existClass)),[],2);

predTest = existClass(activ);

%% Compute the performance metric(s)

% Accuracy
for c = 1:length(unique(cTest))
    ind = find(cTest==c);
    perf.match(c) = sum( predTest(ind)==cTest(ind));
    perf.acc(c)   = perf.match(c)/length(ind) * 100;
    perf.err(c)   = sum( predTest(ind)~=cTest(ind));
end

perf.accuracy   = mean(perf.acc);