function best_cov = estimate_cov(x,minV,maxV,step)

if nargin < 4
minV = 10^-4;
maxV = 10;
step = 128;
end

Nx = size(x,1);
Nd = size(x,2);

distMat = zeros(Nd,Nd);
for d1 = 1 : Nd
    for d2 = d1+1 : Nd
        distMat(d1,d2) = sum((x(:,d1)-x(:,d2)).^2);
    end
end

incr = (maxV-minV)/step;
sigmaAll = [minV:incr:maxV];
llh = zeros(1,length(sigmaAll));
for j = 1:length(sigmaAll)
    sigma = sigmaAll(j);
    llh(j) = compute_llh(distMat,sigma,Nx);
end

plot(llh);
[dummy, i] = max(llh);
best_cov = sigmaAll(i)^2;

function llh = compute_llh(distmat,sigma,Nx)
% Compute LOO estimate for generalization of p(X|\Sigma)
Nd = size(distmat,1);
s1 = -0.5/sigma^2;
s2 = -(Nx/2)*log(2*pi*sigma^2);
smat = s1*distmat;
llh = 0;
for d = 1 : Nd
    llh = llh + s2+log(sum(exp(smat(d,d+1:end)))+sum(exp(smat(1:d-1,d))));
end