
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [log_prob log_prob_full] = log_predictive_dist(data, q_of_z, hp_posterior, hp_prior, opts)
if opts.use_kd_tree
    N = length(data.kdtree);
    D = size(data.kdtree(1).sum_x, 1);
    Na = [data.kdtree(:).N];
    if isequal(opts.algorithm, 'vdp')
        true_Nc = Na*q_of_z; % 1*K
        q_of_z(:,end) = 0;
    end
    Nc = Na*q_of_z; % 1*K
    sum_x = [data.kdtree(:).sum_x] * q_of_z;
else
    [D,N] = size(data.given_data);
    if isequal(opts.algorithm, 'vdp')
        true_Nc = sum(q_of_z, 1); % 1*K
        q_of_z(:,end) = 0;
    end
    Nc = sum(q_of_z, 1); % 1*K
    sum_x = data.given_data * q_of_z;
end
E_pi = mk_E_pi(hp_posterior, hp_prior, opts);
m = (sum_x + repmat(hp_prior.xi0*hp_prior.m0, 1, size(sum_x,2))) ./ repmat(Nc, D, 1);
E_pi(find(Nc==0)) = 0;
n = size(opts.test_data, 2);
log_prob = zeros(size(m,2), n);
for c=1:size(m,2)
    f = hp_posterior.eta(c) + 1 - D;
    sigma = hp_posterior.B{c} * (hp_posterior.xi(c)+1) / hp_posterior.xi(c) / f;
    if E_pi(c) > 0
        log_prob(c,:) = log(E_pi(c)) + log_T_dist(opts.test_data, m(:,c), ...
            sigma, f);
    else
        log_prob(c,:) = -inf;
    end
end
log_prob_full = log_prob;
log_prob = log_sum_exp(log_prob,1);


