
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function E_log_p_of_z_given_other_z = mk_E_log_p_of_z_given_other_z(hp_posterior, hp_prior, opts);
% returns E[log p(z_n|Z^-n)]_q(Z^-n)
% E_log_p_of_z_given_other_z : N by K
q_of_z = hp_posterior.q_of_z;
[N,K] = size(q_of_z);
[E_Nc_minus_n, V_Nc_minus_n] = mk_E_Nc_minus_n(q_of_z); % N by K
use_variance = 1;
if isequal(opts.algorithm, 'cdp')
  if use_variance
    E_log_p_of_z_given_other_z = ...
        log(E_Nc_minus_n + hp_prior.alpha/K) ...
        - 0.5*V_Nc_minus_n./((E_Nc_minus_n+hp_prior.alpha/K).^2) ...
        - log(N - 1 + hp_prior.alpha);
  else
    E_log_p_of_z_given_other_z = ...
        log(E_Nc_minus_n + hp_prior.alpha/K) ...
        - log(N - 1 + hp_prior.alpha);
  end
elseif isequal(opts.algorithm, 'csb')
  E_Nc_minus_n_cumsum_geq = fliplr(cumsum(fliplr(E_Nc_minus_n), 2));
  q_of_z_cumsum_geq = fliplr(cumsum(fliplr(q_of_z), 2));
  dummy = q_of_z_cumsum_geq.*(1-q_of_z_cumsum_geq);
  v_Nc_cumsum_geq = repmat(sum(dummy, 1), N, 1) - dummy;

  E_Nc_minus_n_cumsum = E_Nc_minus_n_cumsum_geq - E_Nc_minus_n;
  q_of_z_cumsum = q_of_z_cumsum_geq - q_of_z;
  dummy = q_of_z_cumsum.*(1-q_of_z_cumsum);
  v_Nc_cumsum = repmat(sum(dummy, 1), N, 1) - dummy;
  
  if use_variance
    first_term = ...
        (log(1+E_Nc_minus_n) ...
         - 0.5*V_Nc_minus_n./((1+E_Nc_minus_n).^2) ...
         - log(1+hp_prior.alpha+E_Nc_minus_n_cumsum_geq) ...
         + 0.5*v_Nc_cumsum_geq./((1+hp_prior.alpha+E_Nc_minus_n_cumsum_geq).^2));
    first_term(:,end) = 0;
    dummy = log(hp_prior.alpha+E_Nc_minus_n_cumsum) ...
            - 0.5*v_Nc_cumsum./((hp_prior.alpha+E_Nc_minus_n_cumsum).^2) ...
            - log(1+hp_prior.alpha+E_Nc_minus_n_cumsum_geq) ...
            + 0.5*v_Nc_cumsum_geq./((1+hp_prior.alpha+E_Nc_minus_n_cumsum_geq).^2);
    second_term = cumsum(dummy, 2) - dummy;
  else
    first_term = log(1+E_Nc_minus_n) - log(1+hp_prior.alpha+E_Nc_minus_n_cumsum_geq);
    first_term(:,end) = 0;
    dummy = log(hp_prior.alpha+E_Nc_minus_n_cumsum) ...
            - log(1+hp_prior.alpha+E_Nc_minus_n_cumsum_geq);
    second_term = cumsum(dummy, 2) - dummy;
  end
  E_log_p_of_z_given_other_z = first_term + second_term;
else
  error('unsupported algorithm');
end
% Gaussian approximation may not give a proper distribution.
% note. E[log p] is not need to be proper

