
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function E_pi = mk_E_pi(hp_posterior, hp_prior, opts);
if isequal(opts.algorithm, 'cdp') | isequal(opts.algorithm, 'non_dp')
  E_pi = hp_posterior.tilde_alpha / sum(hp_posterior.tilde_alpha);
elseif isequal(opts.algorithm, 'bj')
  second_term = [0 cumsum(log(hp_posterior.gamma(2,:)) - log(sum(hp_posterior.gamma)),2)]; % 1 by K
  E_pi = exp([(log(hp_posterior.gamma(1,:)) ...
               - log(sum(hp_posterior.gamma,1)) ...
               + second_term(1:end-1)) ...
              , second_term(end)]); % 1 by K
elseif isequal(opts.algorithm, 'vdp')
  opts_tmp = opts;
  opts_tmp.algorithm = 'bj';
  E_pi = mk_E_pi(hp_posterior, hp_prior, opts_tmp);
  E_pi(end) = max(1 - sum(E_pi(1:end-1)), 0);
elseif isequal(opts.algorithm, 'csb')
  [N,K] = size(hp_posterior.q_of_z);
  a = 1 + hp_posterior.Nc; % 1 by K
  b = hp_prior.alpha + N - cumsum(hp_posterior.Nc, 2); % 1 by K
  first_term = a ./ (a+b);
  first_term(end) = 1;
  second_term = cumprod(b./(a+b)) ./ (b./(a+b));
  E_pi = first_term .* second_term;
else
  error('not supported algorithm')
end

%
% p_TSB(z_test|Z)
%
% \begin{align*}
% p_{\text{TSB}}(Z) =& 
% \alpha^{T-1}
% \prod_{i<T}
% \frac
% {\Gamma(1+N_i) \Gamma(\alpha + N_{>i})}
% {\Gamma(1+\alpha+N_{\geq i})}
% \\
% p_{\text{TSB}}(z_{test}=t,Z) = &
% \left(
% \frac{1+N_t}{1+\alpha+N_{\geq t}}
% \right)^{\mathbb{I}(t<T)}
% \left\{
% \prod_{j<t}\frac
% {\alpha + N_{>j}}
% {1+\alpha+N_{\geq j}}
% \right\}
% \alpha^{T-1}
% \prod_{i<T}
% \frac
% {\Gamma(1+N_i) \Gamma(\alpha + N_{>i})}
% {\Gamma(1+\alpha+N_{\geq i})}
% \\
% p_{\text{TSB}}(z_{test}=t|Z) = &
% \left(
% \frac{1+N_t}{1+\alpha+N_{\geq t}}
% \right)^{\mathbb{I}(t<T)}
% \prod_{j<t}\frac
% {\alpha + N_{>j}}
% {1+\alpha+N_{\geq j}}
% \end{align*}


