function [results] = prior_vdpgm(given_data, opts)
%
% function [results] = vdpgm(given_data, opts)
%

if nargin == 1
    opts = struct();
end
if issparse(given_data)
    given_data = full(given_data);
end
if ~ isfield(opts, 'algorithm')
    % algorithm can be one of 'vdp', 'bj', 'cdp', 'csb' and 'non_dp'
    % vdp : variational DP
    % bj : Blei and Jordan
    % cdp : collapsed Dirichlet prior
    % csb : collapsed stick-breaking
    % non_dp : variational Bayes for Gaussian mixtures
    opts.algorithm = 'vdp';
end
if ~ isfield(opts, 'collapsed_means')
    opts.collapsed_means = 0;
end
if ~ isfield(opts, 'do_sort')
    opts.do_sort = '0';
end
if ~ isfield(opts, 'get_q_of_z')
    opts.get_q_of_z = 0;
end
if ~ isfield(opts, 'weight')
    opts.weight = 1;
end
if ~ isfield(opts, 'get_log_likelihood')
    opts.get_log_likelihood = 0;
end
if ~ isfield(opts, 'use_kd_tree')
    opts.use_kd_tree = 1;
end
if ~ isfield(opts, 'threshold')
    opts.threshold = 1.0e-5;
end
if ~ isfield(opts, 'sis')
    opts.sis = 0;
end
if ~ isfield(opts, 'initial_depth')
    opts.initial_depth = 3;
end
if ~ isfield(opts, 'initial_K')
    opts.initial_K = 1;
end
if ~ isfield(opts, 'ite')
    opts.ite = inf;
end
if ~ isfield(opts, 'do_split')
    opts.do_split = 0;
end
if ~ isfield(opts, 'do_merge')
    opts.do_merge = 0;
end
if ~ isfield(opts, 'do_greedy')
    opts.do_greedy = 1;
end
if ~ isfield(opts, 'max_target_ratio')
    opts.max_target_ratio = 0.5;
end
if ~ isfield(opts, 'init_of_split')
    % 'pc', 'rnd', 'rnd_close' or 'close_f'
    opts.init_of_split = 'pc';
end
if ~ isfield(opts, 'recursive_expanding_depth')
    opts.recursive_expanding_depth = 2;
end
if ~ isfield(opts, 'recursive_expanding_threshold')
    opts.recursive_expanding_threshold = 1.0e-1;
end
if ~ isfield(opts, 'recursive_expanding_frequency')
    opts.recursive_expanding_frequency = 3;
end
% if isfield(opts, 'seed')
%     rand('state', opts.seed);
% else
%     seed = rand('state');
%     results.seed = seed;
% end


data.given_data = given_data;

% the hyperparameters of priors
hp_prior = mk_hp_prior(data, opts);

q_of_z = rand_q_of_z(data, opts.initial_K, opts);

hp_posterior = mk_hp_posterior(data, q_of_z, hp_prior, opts);

results.algorithm = opts.algorithm;
results.hp_prior = hp_prior;
results.hp_posterior = hp_posterior;
results.K = length(hp_posterior.eta);
results.opts = opts;

[results.q_of_z, dummy, dummy] = mk_q_of_z(data, ...
    hp_posterior, ...
    hp_prior, ...
    opts);



