function members = sample_member(dpmm,noMembers,covar_type)

% Sample n finite mixture models from a dpmm


% Initialisation
members       = cell(1,noMembers);
mu            =  dpmm.hp_posterior.m(:,1);
mix.type      = 'gmm';
mix.nin       = length(mu);
mix.ncentres  = length(dpmm.hp_posterior.eta)-1;
if nargin < 3
    mix.covar_type = 'full';
else
    mix.covar_type = covar_type;
end
mix.priors    = dpmm.hp_posterior.Nc(1:mix.ncentres)/ ...
    sum(dpmm.hp_posterior.Nc(1:mix.ncentres));
mix.centres   = zeros(mix.ncentres,mix.nin);
covars    = zeros(mix.nin,mix.nin,mix.ncentres);

DI = cell(1,mix.ncentres);

for n = 1:noMembers
    
    if n == 1
        for nclust = 1:mix.ncentres
            [covars(:,:,nclust), DI{nclust}] = ...
                iwishrnd(dpmm.hp_posterior.B{nclust}, ...
                dpmm.hp_posterior.eta(nclust));
        end
    else
        for nclust = 1:mix.ncentres
            covars(:,:,nclust) = ...
                iwishrnd(dpmm.hp_posterior.B{nclust}, ...
                dpmm.hp_posterior.eta(nclust), DI{nclust});
        end
    end
    
    for nclust = 1:mix.ncentres
        mu    =  dpmm.hp_posterior.m(:,nclust);
        
        mix.centres(nclust,:) = mvnrnd(mu', ...
            covars(:,:,nclust) / dpmm.hp_posterior.xi(nclust));
    end
    
    if strcmp(mix.covar_type,'diag')
        covars_diag = zeros(mix.ncentres,mix.nin);
        for nclust = 1:mix.ncentres
            covars_diag(nclust,:) = diag(covars(:,:,nclust))';
        end
        mix.covars = covars_diag;
    else % full
        mix.covars = covars;
    end
    
    members{n} = mix;
end







