
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [new_data, new_q_of_z, info] = split(c, data, q_of_z, ...
    hp_posterior, hp_prior, opts, ind)
% q_of_z: N*K

new_data = data;

if isequal(opts.splitStrategy, 'bisect_princomp')  % principal eigenvector
    
    arg1_data = new_data.given_data;
    
    dir = divide_by_principal_component(arg1_data, ...
        hp_posterior.B{c}/hp_posterior.eta(c), ...
        hp_posterior.m(:,c));
    q_of_z_c1 = zeros(size(q_of_z,1),1);
    q_of_z_c2 = q_of_z(:,c);
    I = find(dir>=0);
    q_of_z_c1(I) = q_of_z(I,c);
    q_of_z_c2(I) = 0;
elseif isequal(opts.splitStrategy, 'kmeans')
    
    noClust = size(q_of_z,2)-1;
    [dummy clustAssigned] = max(q_of_z(:,1:noClust),[],2);
    
    indc = clustAssigned == c;
    ds.singlets = new_data.given_data(:,indc);
    
    if size(ds.singlets,2) <= 1
        dir_singlet = [];
    else

        if nargin > 6
            id_singlet  = kmeans(ds.singlets',2, ...
                'start',[mean(ds.singlets,2)'; new_data.given_data(:,ind)'], ...
                'emptyaction','singleton');
        else
            id_singlet  = kmeans(ds.singlets',2, ...
                'emptyaction','singleton');
        end
        
        
        dir_singlet = id_singlet;
        dir_singlet(id_singlet==2) = -1;
        
        %         dir_singlet(id_singlet==1) = -1;
        %         dir_singlet(id_singlet==2) = 1;
    end
    
    % Those with direction > 0 can remain in the same cluster
    if any(isnan(dir_singlet)) || isempty(dir_singlet)
        disp('empty cluster in kmeans');
        dir_singlet = ones(size(ds.singlets,2));
    end
    
    dir = ones(size(q_of_z,1),1)*-1;
    dir(indc) = dir_singlet;
    
    q_of_z_c1 = zeros(size(q_of_z,1),1);
    q_of_z_c2 = q_of_z(:,c);
    I = find(dir>=0);
    % Those with direction >= 0 can remain in the same cluster
    q_of_z_c1(I) = q_of_z(I,c);
    % Those with direction < 0 are moved to a new cluster
    q_of_z_c2(I) = 0;
    
else
    error('Error in opts.splitStrategy');
end
new_q_of_z = zeros(size(q_of_z,1), size(q_of_z,2)+1);
new_q_of_z(:,[1:end-2 end]) = q_of_z;
new_q_of_z(:,c) = q_of_z_c1;
new_c = size(new_q_of_z, 2) - 1;
new_q_of_z(:,new_c) = q_of_z_c2;
info.new_c = new_c;

