
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [free_energy, hp_posterior, data, q_of_z, clustClassLabel] = ...
    update_posterior2(data, hp_posterior, hp_prior, ...
    opts, upkdtree, ite, do_sort, ...
    clustClassLabel, classLabel, add_clust)
% update q_of_z: N*K
%disp(['### updating posterior ...'])
free_energy = inf;
if nargin == 4
    upkdtree = 0;
end
if nargin < 6
    ite = inf;
end
if nargin < 7
    do_sort = 1;
end
if nargin > 7
    useConstraint = 1;
else
    useConstraint = 0;
end
if nargin < 10
    add_clust = 1;
end

i = 0;
last_Nc = 0;
start_sort = 0;
while 1
    i = i+1;
    [new_free_energy, log_lambda] = mk_free_energy(data, hp_posterior, hp_prior, opts);
    %disp_status(new_free_energy, hp_posterior, opts);
    if (~isinf(ite) && i>=ite) || ...
            (isinf(ite) && free_energy_improved(free_energy, new_free_energy, 0, opts) == 0)
        free_energy = new_free_energy;
        if do_sort && opts.do_sort && ~ start_sort
            start_sort = 1;
        else
            break
        end
    end
    
    last_Nc = hp_posterior.Nc;
    free_energy = new_free_energy;
    [q_of_z, data] = mk_q_of_z(data, hp_posterior, hp_prior, opts, log_lambda);
    
    freq = opts.recursive_expanding_frequency;
    if opts.use_kd_tree & upkdtree & (freq==1 | mod(i,freq)==1)
        [data, q_of_z] = expand_recursively_until_convergence(data, ...
            q_of_z, ...
            hp_posterior, ...
            hp_prior, opts);
    end
    
    if useConstraint == 1 && ~isempty(classLabel)
        const = bsxfun(@eq, ...
            repmat(clustClassLabel', [length(classLabel) 1]), ...
            classLabel); % get the constraint
        const(find(classLabel==0),:) = ...
            ones(length(find(classLabel==0)),size(const,2)); % set 1 on const for unknown class
        q_of_z = q_of_z + 1e-05;
        q_of_z = q_of_z .* [const ones(length(classLabel),1)];
        q_of_z = bsxfun(@rdivide,q_of_z,sum(q_of_z,2)); % normalise
    end
    
    % check if the last component is small enough
    if isequal(opts.algorithm, 'vdp') && sum(q_of_z(:,end)) > 1.0e-20 && add_clust
        q_of_z(:,end+1) = 0;
    end
    
    if start_sort
        [q_of_z I] = sort_q_of_z(data, q_of_z, opts);
        if useConstraint
            clustClassLabel = clustClassLabel(I(1:end-1));
        end
    end
    
    if isequal(opts.algorithm, 'vdp') && sum(q_of_z(:,end-1)) < 1.0e-10
        q_of_z(:,end-1) = [];
        if useConstraint
            clustClassLabel(end) = [];
        end
    end
    hp_posterior = mk_hp_posterior(data, q_of_z, hp_prior, opts);
end
% disp_status(free_energy, hp_posterior, opts);
%disp(['### updating posterior ... done.'])

