%% Active learning routine

%% Initialisation

% Stop active learning if more than 100 points are not requested
requestLimit      = 100;

% GMM's type of covariance
covar_type = 'full';

% Hyper-parameters
alpha  = 1;
alphaa = 1;
alphab = 1;
betaa  = 1;
betab  = 1;

opts.estimate_alpha = 1;
opts.init_alpha     = 1;
opts.alphaa         = 1;
opts.alphab         = 1;

% Array to store index of selected point
allSelectInd = zeros(min(noDataPool,budget),1);

% Array to store the set of known classes
existClass   = [];
createNewClass = 0;
dpmm = cell(1,1);
classMembers = cell(1,1);

% Array to store the accuracy etc
traceAL.accuracy   = zeros(min(noDataPool,budget),1);
traceAL.noClass    = zeros(min(noDataPool,budget),1);
traceAL.freeEnergy = zeros(min(noDataPool,budget),1);
traceAL.classLabel = zeros(noDataPool,1);
traceAL.alpha      = zeros(noDataPool,1);
traceAL.beta       = zeros(noDataPool,1);
traceAL.score_nan  = [];
traceAL.noSeen     = 0;

% Counters initialisation
count        = 1;
imgCount     = 1;

% Disable warnings
warning_s    = warning('off','stats:kmeans:EmptyCluster');

indexUnlabelled = find(traceAL.classLabel == 0);

if strcmp(sampMethod,'qbc-pyp-s')
    noMembers         = 2;
    beta              = 0.5;
end


%% Estimating priordpmm
optsPrior           = opts;
optsPrior.initial_K = 1;
priordpmm           = prior_vdpgm(dataTrain, optsPrior);

tempData.given_data = dataTrain;
optsPrior           = opts;
optsPrior.initial_K = 1;
[hp_prior covar]    = mk_hp_prior(tempData, optsPrior);

currentDataPool = dataTrain(:,indexUnlabelled);
priormix        = convert_dpmm(priordpmm);
px_new          = gmmprob(priormix,currentDataPool');

%% Stop learning if the pool vanishes or the budget finishes
while noDataPool >= 1 && budget >= 1
    
    currentDataPool = dataTrain(:,indexUnlabelled);
    noCurrentData   = size(currentDataPool,2);
    
    
    % Different sampling strategy
    switch sampMethod
        case 'rand'
            % Select a random point and get its label
            fprintf('Count %d\n',count);
            randIndex       = indexUnlabelled( randperm(noCurrentData) );
            selectInd       = randIndex(1);           % index
            selectPoint     = dataTrain(:,selectInd); % value
            selectLabel     = labelTrain(selectInd);  % class label
            allSelectInd(count) = selectInd;
            
            
        case 'qbc-pyp-s'
            % Query by committee with pwrong measure (stream-based)
            fprintf('Count %d\n',count);
            
            selected    = 0;
            noRequest   = 0;
            noClass     = length(existClass);
            
            while selected ~= 1
                noRequest = noRequest + 1;
                if noRequest > requestLimit || isempty(indexUnlabelled)
                    break;
                end
                
                if noClass < 1
                    randIndex       = indexUnlabelled( randperm(noCurrentData) );
                    selectInd       = randIndex(1);
                    selected        = 1;
                else
                    randIndex       = indexUnlabelled( randperm(noCurrentData) );
                    selectIndTemp   = randIndex(1);     % index
                    selectPointTemp = dataTrain(:,selectIndTemp); % value
                    
                    % Generate class members
                    dpmmMembers = cell(1,noClass);
                    if noMembers == 1
                        for nclass = 1:noClass
                            dpmmMembers{nclass} = convert_dpmm(dpmm{nclass},covar_type);
                        end
                    else
                        for nclass = 1:noClass
                            dpmmMembers{nclass} = sample_member(dpmm{nclass}, ...
                                noMembers,covar_type);
                        end
                    end
                    priormix = convert_dpmm(priordpmm,covar_type);
                    
                    % Compute p_class_given_x_member
                    p_class_given_x_member = cell(1,noMembers);
                    mk_sum = length(setdiff(allSelectInd,0));
                    
                    for nmember = 1:noMembers
                        for nclass = 1:noClass
                            if noMembers == 1
                                mix = dpmmMembers{nclass};
                            else
                                mix = dpmmMembers{nclass}{nmember};
                            end
                            mc = sum(dpmm{nclass}.hp_posterior.Nc);
                            p_class_given_x_member{nmember}(:,nclass) = ...
                                (mc-beta) / (mk_sum + alpha) * ...
                                gmmprob(mix,selectPointTemp');
                        end
                        p_class_given_x_member{nmember}(:,noClass+1) = ...
                            (alpha+beta*noClass) / (mk_sum + alpha) * ...
                            gmmprob(priormix,selectPointTemp');
                        p_class_given_x_member{nmember} = ...
                            normalize(p_class_given_x_member{nmember},2);
                    end
                    
                    % Get votes
                    vote = zeros(1,noMembers);
                    for nmember = 1:noMembers
                        [~, vote(:,nmember)] = max(p_class_given_x_member{nmember}, [],2);
                    end
                    
                    if length(unique(vote)) > 1 || all(vote>noClass)
                        fprintf('Requested\n');
                        selected = 1;
                        selectInd = selectIndTemp;
                    else % if not requested
                        fprintf('Discarded\n');
                        indexUnlabelled(1)     = [];
                        noCurrentData = noCurrentData - 1;
                    end
                    
                end
                traceAL.noSeen = traceAL.noSeen + 1;
            end
            
            if noRequest > requestLimit || isempty(indexUnlabelled)
                break;
            end
            
            selectPoint         = dataTrain(:,selectInd); % value
            selectLabel         = labelTrain(selectInd);  % class label
            allSelectInd(count) = selectInd;
            
            
        otherwise
            error('Invalid active learning method.');
    end
    
    %% Plot the selection
    
    plotActiveSelection;
    
    %% Update the model
    % Update existing classes
    % If not belonging to existing class
    if ~issubset(selectLabel, existClass)
        existClass                       = cat(1, existClass, selectLabel);
        classMembers{length(existClass)} = selectInd;
        createNewClass                   = 1;
    else
        modelIndex                = find(existClass==selectLabel);
        classMembers{modelIndex}  = [classMembers{modelIndex} selectInd];
    end
    traceAL.noClass(count) = length(existClass);
    
    % Update the class label of selected point
    traceAL.classLabel(selectInd) = selectLabel;
    
    % Update the DPMMs
    if createNewClass
        tempData.given_data = selectPoint;
        optsPrior           = opts;
        optsPrior.initial_K = 1;
        optsPrior.xi0       = 0.02;
        hp_prior            = mk_hp_prior(tempData, optsPrior, covar);
        dpmm{length(existClass)} = vdpgm(selectPoint, optsPrior, hp_prior);
        
        dpmm{length(existClass)}.noClust = dpmm{length(existClass)}.K - 1;
        createNewClass = 0; % turns the flag off
    else
        modelIndex       = find(existClass==selectLabel);
        data.given_data  = dataTrain(:,classMembers{modelIndex});
        dpmm{modelIndex} = dpmm_update(dpmm{modelIndex}, data);
    end
    
    if strcmp(sampMethod,'qbc-pyp-s')
        numitemclass = zeros(length(classMembers),1);
        for nnn = 1:length(classMembers)
            numitemclass(nnn) = length(classMembers{nnn});
        end
        [alpha beta] = rand_condisparam(alpha, beta, ...
            length(setdiff(allSelectInd,0)), length(existClass), ...
            alphaa,alphab,betaa,betab,128,128,numitemclass);
        traceAL.alpha(count) = alpha;
        traceAL.beta(count)  = beta;
    end
    
    %% Plot the updated model
    
    plotUpdatedModel;
    
    %% Test data classification
    
    [perf classX] = dpmm_predict(dpmm, classMembers, dataTrain, ...
        dataTest, labelTest, existClass);
    
    traceAL.accuracy(count) = perf.accuracy;
    
    % Plot the classification result
    plotClassificationResult;
    
    %% Update counters
    count           = count + 1;
    budget          = budget - 1;
    if strcmp(sampMethod,'qbc-pyp-s')
        if noClass <= 1
            indexUnlabelled = find(traceAL.classLabel == 0);
        else
            indexUnlabelled(1) = [];
        end
    else
        indexUnlabelled = find(traceAL.classLabel == 0);
    end
    noDataPool      = length(indexUnlabelled);
    
end
