function colour = vsGenPlotColour(n, varargin)
% Generate plots colour
%
% Syntax:  colour = vsGenPlotColour(n, varargin)
%
% Inputs:
%   n         - number of colour to generate
% Optional:
%   colormap  - colour map, e.g. hsv, jet etc. [default=hsv]
%
% Outputs:
%   colour    - [nx3] matrix that specifies that rgb values of colour
%
% Example:
%    colour = vsGenPlotColour(2, 'colormap','jet');
%    plot(rand(1,10),'color',colour(2,:));

% Author: Chen Change Loy (Cavan)
% Created: 20-Apr-2011

%---------------------------- BEGIN CODE ----------------------------------

% Default colours
colour1 = ...
    [79 129 189;
    222 80 76;
    167 215 70;
    178 135 228;
    28 176 28;
    247 143 56;
    144 212 231;
    142 111 111];

colour1 = colour1/255;

% Avoid using some light colours
avoidColour = [ 1 1 0 ; 0 0 1];

if n < 6
    colour = colour1(1:n,:);
    return;
end

% Set default values
cm = 'hsv';

% Read additional inputs
for i=1:2:length(varargin)
    switch varargin{i},
        case 'colormap',
            cm = varargin{i+1};
        otherwise,
            error(['Invalid argument name: ' varargin{i}]);
    end
end

n = n + size(avoidColour,2);
colour2 = eval(strcat('colormap(',cm,'(',num2str(n),'))'));

D = pdist2(colour2,[colour1;avoidColour]);
[y x] = find(D == 0);

colour = [colour1; colour2(setdiff(1:n,y),:) ];

%---------------------------- END OF CODE ---------------------------------